// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.TrafficManager;
using Microsoft.Azure.Management.TrafficManager.Models;

namespace Microsoft.Azure.Management.TrafficManager
{
    /// <summary>
    /// The Windows Azure Traffic Manager management API provides a RESTful set
    /// of web services that interact with Windows Azure Traffic Manager
    /// service for creating, updating, listing, and deleting Traffic Manager
    /// profiles and definitions.  (see
    /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166981.aspx for
    /// more information)
    /// </summary>
    public static partial class ProfileOperationsExtensions
    {
        /// <summary>
        /// Create or update a WATMv2 profile within a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate operation.
        /// </param>
        /// <returns>
        /// The response to a Profile CreateOrUpdate operation.
        /// </returns>
        public static ProfileCreateOrUpdateResponse CreateOrUpdate(this IProfileOperations operations, string resourceGroupName, string profileName, ProfileCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IProfileOperations)s).CreateOrUpdateAsync(resourceGroupName, profileName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create or update a WATMv2 profile within a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate operation.
        /// </param>
        /// <returns>
        /// The response to a Profile CreateOrUpdate operation.
        /// </returns>
        public static Task<ProfileCreateOrUpdateResponse> CreateOrUpdateAsync(this IProfileOperations operations, string resourceGroupName, string profileName, ProfileCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, profileName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a WATMv2 profile within a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IProfileOperations operations, string resourceGroupName, string profileName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IProfileOperations)s).DeleteAsync(resourceGroupName, profileName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes a WATMv2 profile within a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IProfileOperations operations, string resourceGroupName, string profileName)
        {
            return operations.DeleteAsync(resourceGroupName, profileName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets a WATMv2 profile within a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <returns>
        /// The response to a Profile Create operation.
        /// </returns>
        public static ProfileGetResponse Get(this IProfileOperations operations, string resourceGroupName, string profileName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IProfileOperations)s).GetAsync(resourceGroupName, profileName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets a WATMv2 profile within a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <returns>
        /// The response to a Profile Create operation.
        /// </returns>
        public static Task<ProfileGetResponse> GetAsync(this IProfileOperations operations, string resourceGroupName, string profileName)
        {
            return operations.GetAsync(resourceGroupName, profileName, CancellationToken.None);
        }
        
        /// <summary>
        /// Lists all WATMv2 profile within a subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <returns>
        /// The response to a Profile ProfileListAll operation.
        /// </returns>
        public static ProfileListResponse ListAll(this IProfileOperations operations)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IProfileOperations)s).ListAllAsync();
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Lists all WATMv2 profile within a subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <returns>
        /// The response to a Profile ProfileListAll operation.
        /// </returns>
        public static Task<ProfileListResponse> ListAllAsync(this IProfileOperations operations)
        {
            return operations.ListAllAsync(CancellationToken.None);
        }
        
        /// <summary>
        /// Lists all WATMv2 profiles within a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <returns>
        /// The response to a Profile ProfileListAll operation.
        /// </returns>
        public static ProfileListResponse ListAllInResourceGroup(this IProfileOperations operations, string resourceGroupName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IProfileOperations)s).ListAllInResourceGroupAsync(resourceGroupName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Lists all WATMv2 profiles within a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IProfileOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <returns>
        /// The response to a Profile ProfileListAll operation.
        /// </returns>
        public static Task<ProfileListResponse> ListAllInResourceGroupAsync(this IProfileOperations operations, string resourceGroupName)
        {
            return operations.ListAllInResourceGroupAsync(resourceGroupName, CancellationToken.None);
        }
    }
}
