// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.

namespace Microsoft.Azure.Search.Models
{
    using System;
    using System.Linq;
    using System.Collections.Generic;
    using Newtonsoft.Json;
    using Microsoft.Rest;
    using Microsoft.Rest.Serialization;
    using Microsoft.Rest.Azure;

    /// <summary>
    /// Status of an indexing operation for a single document.
    /// </summary>
    public class IndexingResult
    {
        /// <summary>
        /// Initializes a new instance of the IndexingResult class.
        /// </summary>
        public IndexingResult() { }

        /// <summary>
        /// Initializes a new instance of the IndexingResult class.
        /// </summary>
        public IndexingResult(string key = default(string), string errorMessage = default(string), bool succeeded = default(bool))
        {
            Key = key;
            ErrorMessage = errorMessage;
            Succeeded = succeeded;
        }

        /// <summary>
        /// Gets the key of a document that was in the indexing request.
        /// </summary>
        [JsonProperty(PropertyName = "key")]
        public string Key { get; private set; }

        /// <summary>
        /// Gets the error message explaining why the indexing operation
        /// failed for the document identified by Key; null if Succeeded is
        /// true.
        /// </summary>
        [JsonProperty(PropertyName = "errorMessage")]
        public string ErrorMessage { get; private set; }

        /// <summary>
        /// Gets a value indicating whether the indexing operation succeeded for the document identified by 
        /// <c cref="Microsoft.Azure.Search.Models.IndexingResult.Key">Key</c>.
        /// </summary>
        [JsonProperty(PropertyName = "status")]
        public bool Succeeded { get; private set; }

    }
}
