// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Xml;
using Hyak.Common;
using Hyak.Common.Internals;
using Microsoft.Azure;
using Microsoft.WindowsAzure.Management.Monitoring.Autoscale;
using Microsoft.WindowsAzure.Management.Monitoring.Autoscale.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.WindowsAzure.Management.Monitoring.Autoscale
{
    /// <summary>
    /// Operations for managing the autoscale settings.
    /// </summary>
    internal partial class SettingOperations : IServiceOperations<AutoscaleClient>, ISettingOperations
    {
        /// <summary>
        /// Initializes a new instance of the SettingOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal SettingOperations(AutoscaleClient client)
        {
            this._client = client;
        }
        
        private AutoscaleClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.WindowsAzure.Management.Monitoring.Autoscale.AutoscaleClient.
        /// </summary>
        public AutoscaleClient Client
        {
            get { return this._client; }
        }
        
        /// <param name='resourceId'>
        /// Required. The resource ID.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<AzureOperationResponse> CreateOrUpdateAsync(string resourceId, AutoscaleSettingCreateOrUpdateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceId == null)
            {
                throw new ArgumentNullException("resourceId");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceId", resourceId);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "CreateOrUpdateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/monitoring/autoscalesettings";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("resourceId=" + Uri.EscapeDataString(resourceId));
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject autoscaleSettingCreateOrUpdateParametersValue = new JObject();
                requestDoc = autoscaleSettingCreateOrUpdateParametersValue;
                
                if (parameters.Setting != null)
                {
                    if (parameters.Setting.Profiles != null)
                    {
                        if (parameters.Setting.Profiles is ILazyCollection == false || ((ILazyCollection)parameters.Setting.Profiles).IsInitialized)
                        {
                            JArray profilesArray = new JArray();
                            foreach (AutoscaleProfile profilesItem in parameters.Setting.Profiles)
                            {
                                JObject autoscaleProfileValue = new JObject();
                                profilesArray.Add(autoscaleProfileValue);
                                
                                if (profilesItem.Name != null)
                                {
                                    autoscaleProfileValue["Name"] = profilesItem.Name;
                                }
                                
                                if (profilesItem.Capacity != null)
                                {
                                    JObject capacityValue = new JObject();
                                    autoscaleProfileValue["Capacity"] = capacityValue;
                                    
                                    if (profilesItem.Capacity.Minimum != null)
                                    {
                                        capacityValue["Minimum"] = profilesItem.Capacity.Minimum;
                                    }
                                    
                                    if (profilesItem.Capacity.Maximum != null)
                                    {
                                        capacityValue["Maximum"] = profilesItem.Capacity.Maximum;
                                    }
                                    
                                    if (profilesItem.Capacity.Default != null)
                                    {
                                        capacityValue["Default"] = profilesItem.Capacity.Default;
                                    }
                                }
                                
                                if (profilesItem.Rules != null)
                                {
                                    if (profilesItem.Rules is ILazyCollection == false || ((ILazyCollection)profilesItem.Rules).IsInitialized)
                                    {
                                        JArray rulesArray = new JArray();
                                        foreach (ScaleRule rulesItem in profilesItem.Rules)
                                        {
                                            JObject scaleRuleValue = new JObject();
                                            rulesArray.Add(scaleRuleValue);
                                            
                                            if (rulesItem.MetricTrigger != null)
                                            {
                                                JObject metricTriggerValue = new JObject();
                                                scaleRuleValue["MetricTrigger"] = metricTriggerValue;
                                                
                                                if (rulesItem.MetricTrigger.MetricName != null)
                                                {
                                                    metricTriggerValue["MetricName"] = rulesItem.MetricTrigger.MetricName;
                                                }
                                                
                                                if (rulesItem.MetricTrigger.MetricNamespace != null)
                                                {
                                                    metricTriggerValue["MetricNamespace"] = rulesItem.MetricTrigger.MetricNamespace;
                                                }
                                                
                                                if (rulesItem.MetricTrigger.MetricSource != null)
                                                {
                                                    metricTriggerValue["MetricSource"] = rulesItem.MetricTrigger.MetricSource;
                                                }
                                                
                                                metricTriggerValue["TimeGrain"] = XmlConvert.ToString(rulesItem.MetricTrigger.TimeGrain);
                                                
                                                metricTriggerValue["Statistic"] = rulesItem.MetricTrigger.Statistic.ToString();
                                                
                                                metricTriggerValue["TimeWindow"] = XmlConvert.ToString(rulesItem.MetricTrigger.TimeWindow);
                                                
                                                metricTriggerValue["TimeAggregation"] = rulesItem.MetricTrigger.TimeAggregation.ToString();
                                                
                                                metricTriggerValue["Operator"] = rulesItem.MetricTrigger.Operator.ToString();
                                                
                                                metricTriggerValue["Threshold"] = rulesItem.MetricTrigger.Threshold;
                                            }
                                            
                                            if (rulesItem.ScaleAction != null)
                                            {
                                                JObject scaleActionValue = new JObject();
                                                scaleRuleValue["ScaleAction"] = scaleActionValue;
                                                
                                                scaleActionValue["Direction"] = rulesItem.ScaleAction.Direction.ToString();
                                                
                                                scaleActionValue["Type"] = rulesItem.ScaleAction.Type.ToString();
                                                
                                                if (rulesItem.ScaleAction.Value != null)
                                                {
                                                    scaleActionValue["Value"] = rulesItem.ScaleAction.Value;
                                                }
                                                
                                                scaleActionValue["Cooldown"] = XmlConvert.ToString(rulesItem.ScaleAction.Cooldown);
                                            }
                                        }
                                        autoscaleProfileValue["Rules"] = rulesArray;
                                    }
                                }
                                
                                if (profilesItem.FixedDate != null)
                                {
                                    JObject fixedDateValue = new JObject();
                                    autoscaleProfileValue["FixedDate"] = fixedDateValue;
                                    
                                    if (profilesItem.FixedDate.TimeZone != null)
                                    {
                                        fixedDateValue["TimeZone"] = profilesItem.FixedDate.TimeZone;
                                    }
                                    
                                    fixedDateValue["Start"] = profilesItem.FixedDate.Start;
                                    
                                    fixedDateValue["End"] = profilesItem.FixedDate.End;
                                }
                                
                                if (profilesItem.Recurrence != null)
                                {
                                    JObject recurrenceValue = new JObject();
                                    autoscaleProfileValue["Recurrence"] = recurrenceValue;
                                    
                                    recurrenceValue["Frequency"] = profilesItem.Recurrence.Frequency.ToString();
                                    
                                    if (profilesItem.Recurrence.Schedule != null)
                                    {
                                        JObject scheduleValue = new JObject();
                                        recurrenceValue["Schedule"] = scheduleValue;
                                        
                                        if (profilesItem.Recurrence.Schedule.TimeZone != null)
                                        {
                                            scheduleValue["TimeZone"] = profilesItem.Recurrence.Schedule.TimeZone;
                                        }
                                        
                                        if (profilesItem.Recurrence.Schedule.Days != null)
                                        {
                                            if (profilesItem.Recurrence.Schedule.Days is ILazyCollection == false || ((ILazyCollection)profilesItem.Recurrence.Schedule.Days).IsInitialized)
                                            {
                                                JArray daysArray = new JArray();
                                                foreach (string daysItem in profilesItem.Recurrence.Schedule.Days)
                                                {
                                                    daysArray.Add(daysItem);
                                                }
                                                scheduleValue["Days"] = daysArray;
                                            }
                                        }
                                        
                                        if (profilesItem.Recurrence.Schedule.Hours != null)
                                        {
                                            if (profilesItem.Recurrence.Schedule.Hours is ILazyCollection == false || ((ILazyCollection)profilesItem.Recurrence.Schedule.Hours).IsInitialized)
                                            {
                                                JArray hoursArray = new JArray();
                                                foreach (int hoursItem in profilesItem.Recurrence.Schedule.Hours)
                                                {
                                                    hoursArray.Add(hoursItem);
                                                }
                                                scheduleValue["Hours"] = hoursArray;
                                            }
                                        }
                                        
                                        if (profilesItem.Recurrence.Schedule.Minutes != null)
                                        {
                                            if (profilesItem.Recurrence.Schedule.Minutes is ILazyCollection == false || ((ILazyCollection)profilesItem.Recurrence.Schedule.Minutes).IsInitialized)
                                            {
                                                JArray minutesArray = new JArray();
                                                foreach (int minutesItem in profilesItem.Recurrence.Schedule.Minutes)
                                                {
                                                    minutesArray.Add(minutesItem);
                                                }
                                                scheduleValue["Minutes"] = minutesArray;
                                            }
                                        }
                                    }
                                }
                            }
                            autoscaleSettingCreateOrUpdateParametersValue["Profiles"] = profilesArray;
                        }
                    }
                    
                    autoscaleSettingCreateOrUpdateParametersValue["Enabled"] = parameters.Setting.Enabled;
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json; charset=utf-8");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    AzureOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new AzureOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <param name='resourceId'>
        /// Required. The resource ID.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<AzureOperationResponse> DeleteAsync(string resourceId, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceId == null)
            {
                throw new ArgumentNullException("resourceId");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceId", resourceId);
                TracingAdapter.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/monitoring/autoscalesettings";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("resourceId=" + Uri.EscapeDataString(resourceId));
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    AzureOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new AzureOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <param name='resourceId'>
        /// Required. The resource ID.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<AutoscaleSettingGetResponse> GetAsync(string resourceId, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceId == null)
            {
                throw new ArgumentNullException("resourceId");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceId", resourceId);
                TracingAdapter.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/monitoring/autoscalesettings";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("resourceId=" + Uri.EscapeDataString(resourceId));
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept", "application/json");
                httpRequest.Headers.Add("x-ms-version", "2013-10-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    AutoscaleSettingGetResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new AutoscaleSettingGetResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            AutoscaleSetting settingInstance = new AutoscaleSetting();
                            result.Setting = settingInstance;
                            
                            JToken profilesArray = responseDoc["Profiles"];
                            if (profilesArray != null && profilesArray.Type != JTokenType.Null)
                            {
                                foreach (JToken profilesValue in ((JArray)profilesArray))
                                {
                                    AutoscaleProfile autoscaleProfileInstance = new AutoscaleProfile();
                                    settingInstance.Profiles.Add(autoscaleProfileInstance);
                                    
                                    JToken nameValue = profilesValue["Name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        autoscaleProfileInstance.Name = nameInstance;
                                    }
                                    
                                    JToken capacityValue = profilesValue["Capacity"];
                                    if (capacityValue != null && capacityValue.Type != JTokenType.Null)
                                    {
                                        ScaleCapacity capacityInstance = new ScaleCapacity();
                                        autoscaleProfileInstance.Capacity = capacityInstance;
                                        
                                        JToken minimumValue = capacityValue["Minimum"];
                                        if (minimumValue != null && minimumValue.Type != JTokenType.Null)
                                        {
                                            string minimumInstance = ((string)minimumValue);
                                            capacityInstance.Minimum = minimumInstance;
                                        }
                                        
                                        JToken maximumValue = capacityValue["Maximum"];
                                        if (maximumValue != null && maximumValue.Type != JTokenType.Null)
                                        {
                                            string maximumInstance = ((string)maximumValue);
                                            capacityInstance.Maximum = maximumInstance;
                                        }
                                        
                                        JToken defaultValue = capacityValue["Default"];
                                        if (defaultValue != null && defaultValue.Type != JTokenType.Null)
                                        {
                                            string defaultInstance = ((string)defaultValue);
                                            capacityInstance.Default = defaultInstance;
                                        }
                                    }
                                    
                                    JToken rulesArray = profilesValue["Rules"];
                                    if (rulesArray != null && rulesArray.Type != JTokenType.Null)
                                    {
                                        foreach (JToken rulesValue in ((JArray)rulesArray))
                                        {
                                            ScaleRule scaleRuleInstance = new ScaleRule();
                                            autoscaleProfileInstance.Rules.Add(scaleRuleInstance);
                                            
                                            JToken metricTriggerValue = rulesValue["MetricTrigger"];
                                            if (metricTriggerValue != null && metricTriggerValue.Type != JTokenType.Null)
                                            {
                                                MetricTrigger metricTriggerInstance = new MetricTrigger();
                                                scaleRuleInstance.MetricTrigger = metricTriggerInstance;
                                                
                                                JToken metricNameValue = metricTriggerValue["MetricName"];
                                                if (metricNameValue != null && metricNameValue.Type != JTokenType.Null)
                                                {
                                                    string metricNameInstance = ((string)metricNameValue);
                                                    metricTriggerInstance.MetricName = metricNameInstance;
                                                }
                                                
                                                JToken metricNamespaceValue = metricTriggerValue["MetricNamespace"];
                                                if (metricNamespaceValue != null && metricNamespaceValue.Type != JTokenType.Null)
                                                {
                                                    string metricNamespaceInstance = ((string)metricNamespaceValue);
                                                    metricTriggerInstance.MetricNamespace = metricNamespaceInstance;
                                                }
                                                
                                                JToken metricSourceValue = metricTriggerValue["MetricSource"];
                                                if (metricSourceValue != null && metricSourceValue.Type != JTokenType.Null)
                                                {
                                                    string metricSourceInstance = ((string)metricSourceValue);
                                                    metricTriggerInstance.MetricSource = metricSourceInstance;
                                                }
                                                
                                                JToken timeGrainValue = metricTriggerValue["TimeGrain"];
                                                if (timeGrainValue != null && timeGrainValue.Type != JTokenType.Null)
                                                {
                                                    TimeSpan timeGrainInstance = XmlConvert.ToTimeSpan(((string)timeGrainValue));
                                                    metricTriggerInstance.TimeGrain = timeGrainInstance;
                                                }
                                                
                                                JToken statisticValue = metricTriggerValue["Statistic"];
                                                if (statisticValue != null && statisticValue.Type != JTokenType.Null)
                                                {
                                                    MetricStatisticType statisticInstance = ((MetricStatisticType)Enum.Parse(typeof(MetricStatisticType), ((string)statisticValue), true));
                                                    metricTriggerInstance.Statistic = statisticInstance;
                                                }
                                                
                                                JToken timeWindowValue = metricTriggerValue["TimeWindow"];
                                                if (timeWindowValue != null && timeWindowValue.Type != JTokenType.Null)
                                                {
                                                    TimeSpan timeWindowInstance = XmlConvert.ToTimeSpan(((string)timeWindowValue));
                                                    metricTriggerInstance.TimeWindow = timeWindowInstance;
                                                }
                                                
                                                JToken timeAggregationValue = metricTriggerValue["TimeAggregation"];
                                                if (timeAggregationValue != null && timeAggregationValue.Type != JTokenType.Null)
                                                {
                                                    TimeAggregationType timeAggregationInstance = ((TimeAggregationType)Enum.Parse(typeof(TimeAggregationType), ((string)timeAggregationValue), true));
                                                    metricTriggerInstance.TimeAggregation = timeAggregationInstance;
                                                }
                                                
                                                JToken operatorValue = metricTriggerValue["Operator"];
                                                if (operatorValue != null && operatorValue.Type != JTokenType.Null)
                                                {
                                                    ComparisonOperationType operatorInstance = ((ComparisonOperationType)Enum.Parse(typeof(ComparisonOperationType), ((string)operatorValue), true));
                                                    metricTriggerInstance.Operator = operatorInstance;
                                                }
                                                
                                                JToken thresholdValue = metricTriggerValue["Threshold"];
                                                if (thresholdValue != null && thresholdValue.Type != JTokenType.Null)
                                                {
                                                    double thresholdInstance = ((double)thresholdValue);
                                                    metricTriggerInstance.Threshold = thresholdInstance;
                                                }
                                            }
                                            
                                            JToken scaleActionValue = rulesValue["ScaleAction"];
                                            if (scaleActionValue != null && scaleActionValue.Type != JTokenType.Null)
                                            {
                                                ScaleAction scaleActionInstance = new ScaleAction();
                                                scaleRuleInstance.ScaleAction = scaleActionInstance;
                                                
                                                JToken directionValue = scaleActionValue["Direction"];
                                                if (directionValue != null && directionValue.Type != JTokenType.Null)
                                                {
                                                    ScaleDirection directionInstance = ((ScaleDirection)Enum.Parse(typeof(ScaleDirection), ((string)directionValue), true));
                                                    scaleActionInstance.Direction = directionInstance;
                                                }
                                                
                                                JToken typeValue = scaleActionValue["Type"];
                                                if (typeValue != null && typeValue.Type != JTokenType.Null)
                                                {
                                                    ScaleType typeInstance = ((ScaleType)Enum.Parse(typeof(ScaleType), ((string)typeValue), true));
                                                    scaleActionInstance.Type = typeInstance;
                                                }
                                                
                                                JToken valueValue = scaleActionValue["Value"];
                                                if (valueValue != null && valueValue.Type != JTokenType.Null)
                                                {
                                                    string valueInstance = ((string)valueValue);
                                                    scaleActionInstance.Value = valueInstance;
                                                }
                                                
                                                JToken cooldownValue = scaleActionValue["Cooldown"];
                                                if (cooldownValue != null && cooldownValue.Type != JTokenType.Null)
                                                {
                                                    TimeSpan cooldownInstance = XmlConvert.ToTimeSpan(((string)cooldownValue));
                                                    scaleActionInstance.Cooldown = cooldownInstance;
                                                }
                                            }
                                        }
                                    }
                                    
                                    JToken fixedDateValue = profilesValue["FixedDate"];
                                    if (fixedDateValue != null && fixedDateValue.Type != JTokenType.Null)
                                    {
                                        TimeWindow fixedDateInstance = new TimeWindow();
                                        autoscaleProfileInstance.FixedDate = fixedDateInstance;
                                        
                                        JToken timeZoneValue = fixedDateValue["TimeZone"];
                                        if (timeZoneValue != null && timeZoneValue.Type != JTokenType.Null)
                                        {
                                            string timeZoneInstance = ((string)timeZoneValue);
                                            fixedDateInstance.TimeZone = timeZoneInstance;
                                        }
                                        
                                        JToken startValue = fixedDateValue["Start"];
                                        if (startValue != null && startValue.Type != JTokenType.Null)
                                        {
                                            DateTime startInstance = ((DateTime)startValue);
                                            fixedDateInstance.Start = startInstance;
                                        }
                                        
                                        JToken endValue = fixedDateValue["End"];
                                        if (endValue != null && endValue.Type != JTokenType.Null)
                                        {
                                            DateTime endInstance = ((DateTime)endValue);
                                            fixedDateInstance.End = endInstance;
                                        }
                                    }
                                    
                                    JToken recurrenceValue = profilesValue["Recurrence"];
                                    if (recurrenceValue != null && recurrenceValue.Type != JTokenType.Null)
                                    {
                                        Recurrence recurrenceInstance = new Recurrence();
                                        autoscaleProfileInstance.Recurrence = recurrenceInstance;
                                        
                                        JToken frequencyValue = recurrenceValue["Frequency"];
                                        if (frequencyValue != null && frequencyValue.Type != JTokenType.Null)
                                        {
                                            RecurrenceFrequency frequencyInstance = ((RecurrenceFrequency)Enum.Parse(typeof(RecurrenceFrequency), ((string)frequencyValue), true));
                                            recurrenceInstance.Frequency = frequencyInstance;
                                        }
                                        
                                        JToken scheduleValue = recurrenceValue["Schedule"];
                                        if (scheduleValue != null && scheduleValue.Type != JTokenType.Null)
                                        {
                                            RecurrentSchedule scheduleInstance = new RecurrentSchedule();
                                            recurrenceInstance.Schedule = scheduleInstance;
                                            
                                            JToken timeZoneValue2 = scheduleValue["TimeZone"];
                                            if (timeZoneValue2 != null && timeZoneValue2.Type != JTokenType.Null)
                                            {
                                                string timeZoneInstance2 = ((string)timeZoneValue2);
                                                scheduleInstance.TimeZone = timeZoneInstance2;
                                            }
                                            
                                            JToken daysArray = scheduleValue["Days"];
                                            if (daysArray != null && daysArray.Type != JTokenType.Null)
                                            {
                                                foreach (JToken daysValue in ((JArray)daysArray))
                                                {
                                                    scheduleInstance.Days.Add(((string)daysValue));
                                                }
                                            }
                                            
                                            JToken hoursArray = scheduleValue["Hours"];
                                            if (hoursArray != null && hoursArray.Type != JTokenType.Null)
                                            {
                                                foreach (JToken hoursValue in ((JArray)hoursArray))
                                                {
                                                    scheduleInstance.Hours.Add(((int)hoursValue));
                                                }
                                            }
                                            
                                            JToken minutesArray = scheduleValue["Minutes"];
                                            if (minutesArray != null && minutesArray.Type != JTokenType.Null)
                                            {
                                                foreach (JToken minutesValue in ((JArray)minutesArray))
                                                {
                                                    scheduleInstance.Minutes.Add(((int)minutesValue));
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            
                            JToken enabledValue = responseDoc["Enabled"];
                            if (enabledValue != null && enabledValue.Type != JTokenType.Null)
                            {
                                bool enabledInstance = ((bool)enabledValue);
                                settingInstance.Enabled = enabledInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
