// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.WindowsAzure.WebSitesExtensions;
using Microsoft.WindowsAzure.WebSitesExtensions.Models;

namespace Microsoft.WindowsAzure.WebSitesExtensions
{
    /// <summary>
    /// The websites extensions client manages the web sites deployments, web
    /// jobs and other extensions.
    /// </summary>
    public static partial class ContinuousWebJobOperationsExtensions
    {
        /// <summary>
        /// Delete a continuous job.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IContinuousWebJobOperations operations, string jobName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IContinuousWebJobOperations)s).DeleteAsync(jobName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Delete a continuous job.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IContinuousWebJobOperations operations, string jobName)
        {
            return operations.DeleteAsync(jobName, CancellationToken.None);
        }
        
        /// <summary>
        /// Get a continuous WebJob.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <returns>
        /// The get continuous WebJob Operation Response.
        /// </returns>
        public static ContinuousWebJobGetResponse Get(this IContinuousWebJobOperations operations, string jobName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IContinuousWebJobOperations)s).GetAsync(jobName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Get a continuous WebJob.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <returns>
        /// The get continuous WebJob Operation Response.
        /// </returns>
        public static Task<ContinuousWebJobGetResponse> GetAsync(this IContinuousWebJobOperations operations, string jobName)
        {
            return operations.GetAsync(jobName, CancellationToken.None);
        }
        
        /// <summary>
        /// Get the settings of a continuous WebJob.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <returns>
        /// The continuous WebJob settings operation response.
        /// </returns>
        public static ContinuousWebJobSettingsResponse GetSettings(this IContinuousWebJobOperations operations, string jobName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IContinuousWebJobOperations)s).GetSettingsAsync(jobName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Get the settings of a continuous WebJob.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <returns>
        /// The continuous WebJob settings operation response.
        /// </returns>
        public static Task<ContinuousWebJobSettingsResponse> GetSettingsAsync(this IContinuousWebJobOperations operations, string jobName)
        {
            return operations.GetSettingsAsync(jobName, CancellationToken.None);
        }
        
        /// <summary>
        /// List the continuous web jobs.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <returns>
        /// The list of continuous WebJobs operation response.
        /// </returns>
        public static ContinuousWebJobListResponse List(this IContinuousWebJobOperations operations)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IContinuousWebJobOperations)s).ListAsync();
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// List the continuous web jobs.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <returns>
        /// The list of continuous WebJobs operation response.
        /// </returns>
        public static Task<ContinuousWebJobListResponse> ListAsync(this IContinuousWebJobOperations operations)
        {
            return operations.ListAsync(CancellationToken.None);
        }
        
        /// <summary>
        /// Set the settings of a continuous WebJob (will replace the current
        /// settings of the WebJob).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <param name='settings'>
        /// Required. The continuous WebJob settings.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse SetSettings(this IContinuousWebJobOperations operations, string jobName, ContinuousWebJobSettingsUpdateParameters settings)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IContinuousWebJobOperations)s).SetSettingsAsync(jobName, settings);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Set the settings of a continuous WebJob (will replace the current
        /// settings of the WebJob).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <param name='settings'>
        /// Required. The continuous WebJob settings.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> SetSettingsAsync(this IContinuousWebJobOperations operations, string jobName, ContinuousWebJobSettingsUpdateParameters settings)
        {
            return operations.SetSettingsAsync(jobName, settings, CancellationToken.None);
        }
        
        /// <summary>
        /// Start a continuous WebJob.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The WebJob name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Start(this IContinuousWebJobOperations operations, string jobName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IContinuousWebJobOperations)s).StartAsync(jobName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Start a continuous WebJob.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The WebJob name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> StartAsync(this IContinuousWebJobOperations operations, string jobName)
        {
            return operations.StartAsync(jobName, CancellationToken.None);
        }
        
        /// <summary>
        /// Stop a continuous WebJob.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The WebJob name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Stop(this IContinuousWebJobOperations operations, string jobName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IContinuousWebJobOperations)s).StopAsync(jobName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Stop a continuous WebJob.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The WebJob name.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> StopAsync(this IContinuousWebJobOperations operations, string jobName)
        {
            return operations.StopAsync(jobName, CancellationToken.None);
        }
        
        /// <summary>
        /// Create or replace a continuous WebJob with a script file (.exe,
        /// .bat, .php, .js...).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <param name='fileName'>
        /// Required. The file name.
        /// </param>
        /// <param name='jobContent'>
        /// Required. The file content.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse UploadFile(this IContinuousWebJobOperations operations, string jobName, string fileName, Stream jobContent)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IContinuousWebJobOperations)s).UploadFileAsync(jobName, fileName, jobContent);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create or replace a continuous WebJob with a script file (.exe,
        /// .bat, .php, .js...).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <param name='fileName'>
        /// Required. The file name.
        /// </param>
        /// <param name='jobContent'>
        /// Required. The file content.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> UploadFileAsync(this IContinuousWebJobOperations operations, string jobName, string fileName, Stream jobContent)
        {
            return operations.UploadFileAsync(jobName, fileName, jobContent, CancellationToken.None);
        }
        
        /// <summary>
        /// Create or replace a continuous WebJob with a zip file (containing
        /// the WebJob binaries).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <param name='fileName'>
        /// Required. The zip file name.
        /// </param>
        /// <param name='jobContent'>
        /// Required. The zip file content.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse UploadZip(this IContinuousWebJobOperations operations, string jobName, string fileName, Stream jobContent)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IContinuousWebJobOperations)s).UploadZipAsync(jobName, fileName, jobContent);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create or replace a continuous WebJob with a zip file (containing
        /// the WebJob binaries).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.WebSitesExtensions.IContinuousWebJobOperations.
        /// </param>
        /// <param name='jobName'>
        /// Required. The continuous WebJob name.
        /// </param>
        /// <param name='fileName'>
        /// Required. The zip file name.
        /// </param>
        /// <param name='jobContent'>
        /// Required. The zip file content.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> UploadZipAsync(this IContinuousWebJobOperations operations, string jobName, string fileName, Stream jobContent)
        {
            return operations.UploadZipAsync(jobName, fileName, jobContent, CancellationToken.None);
        }
    }
}
